
#import <Cocoa/Cocoa.h>
#import "PreferenceController.h"

// keys into the UserDefaults

NSString *BWMessageKey = @"Message";
NSString *BWAlertTimesKey = @"Alert Times";



@implementation PreferenceController


// reset the prefs to the factory default - alert on 10 minutes til the
// hour, and use the 'you spaz' message

- (NSMutableDictionary *) makeFactoryDefaults
{
    NSMutableDictionary *defaultValues = [NSMutableDictionary dictionary];
    
    [defaultValues setObject: @"go for a walk around the block you spaz" 
                        forKey: BWMessageKey];

    {
	NSMutableSet *defaultTimes;
        NSData *data;

	// 5 minute increments
        defaultTimes = [NSMutableSet setWithCapacity: 12];

	// default to 0:50
        [defaultTimes addObject: [NSNumber numberWithInt: 50]];

        data = [NSArchiver archivedDataWithRootObject: defaultTimes];
        [defaultValues setObject: data forKey: BWAlertTimesKey];
    }
   
    return (defaultValues);
    
} // makeFactoryDefaults




// change the set of alert times in the preference controller, using the
// prefs as the source of the data

- (void) setAlertTimesFromPrefs
{
    NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
    NSData *alertTimesAsData;

    [alertTimes release];

    alertTimesAsData = [defaults objectForKey: BWAlertTimesKey];
    alertTimes = [NSUnarchiver unarchiveObjectWithData: alertTimesAsData];

    [alertTimes retain];
    
} // setAlertTimesFromPrefs




// initialize ourselves

- (id) init
{
    if (self = [super initWithWindowNibName: @"Preferences"]) {
        NSMutableDictionary *defaultValues = [self makeFactoryDefaults];

        [[NSUserDefaults standardUserDefaults] registerDefaults: defaultValues];

	[self setAlertTimesFromPrefs];
	
        [self setWindowFrameAutosaveName: @"PreferenceWindow"];
    }
    
    return (self);
    
} // init




// the updateUI paradigm from Hillegass.  Something Has Changed, so bash
// all the UI elements to match

- (void) updateUI
{
    [message setStringValue: [self alertMessage]];
    
    // walk the box turning things on
    {
        int tags[] = { 0, 5, 10, 15, 20, 25, 30, 35, 40, 45, 50, 55 };
        int i;
        
        for (i = 0; i < sizeof(tags) / sizeof(int); i++) {
            int tag = tags[i];
            NSButton *button = [hoursBox viewWithTag: tag];
                        
            if ([alertTimes containsObject: [NSNumber numberWithInt: tag]]) {
                [button setState: YES];
            } else {
                [button setState: NO];
            }
        }
     }

} // updateUI




// update the UI once we get the pref window loaded

- (void) windowDidLoad
{
    [self updateUI];    

} // windowDidLoad




// handle a checkbox click

- (IBAction) changeAlertTime: (id) sender
{
    // the checkbox's tag has the number of minutes after the hour the 
    // alert should be shown

    int tag = [sender tag];
    BOOL onOff = [sender state];
    
    if (onOff) {
        [alertTimes addObject: [NSNumber numberWithInt: tag]];
    } else {
        [alertTimes removeObject: [NSNumber numberWithInt: tag]];
    }

    // and store into the defaults
    {
        NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
        NSData *data = [NSArchiver archivedDataWithRootObject: alertTimes];
        [defaults setObject: data forKey: BWAlertTimesKey];
    }
        
} // changeAlertTime




// user wants a different alert message

- (IBAction) changeAlertMessage: (id) sender
{
    NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
    
    [defaults setObject: [sender stringValue] forKey: BWMessageKey];

} // changeAlertMessage




// handle the factory defaulst button

- (IBAction) factoryDefaults: (id) sender
{
    NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];

    // remove our current values
    [defaults removeObjectForKey: BWMessageKey];
    [defaults removeObjectForKey: BWAlertTimesKey];

    // reset the defaults
    // !!! verify we actually have to do this since we've already set the
    // !!! defaults when we started up
    [self makeFactoryDefaults];
    
    // and set up our alertTimes field
    [self setAlertTimesFromPrefs];
    
    [self updateUI];

} // factoryDefaults




// get the 'go take a walk you spaz' message

- (NSString *) alertMessage
{
    NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
    NSString *string = [defaults objectForKey: BWMessageKey];
    return (string);

} // alertMessage




// get the times (NSNumbers) to alert the user

- (NSSet *) alertTimes
{
    return (alertTimes);
} // alertTimes




// clean up our mess

- (void) dealloc
{
    [[NSUserDefaults standardUserDefaults] synchronize];
    [super dealloc];
} // dealloc


@end // PreferenceController
